from bzv.simplejson.decoder import JSONDecoder
from bzv.simplejson.encoder import JSONEncoder
from cgi import parse_qs
from cqlparser import parseString as parseCQL
from merescocore.framework import Observable
from urlparse import urlsplit
from httplib import HTTPConnection, OK
import urllib

def getDrilldownFields(drilldownConfig,drilldownFieldsName):
    # no access to drilldownConfig, why?
#    logger.debug("drilldownConfig:" + str(drilldownConfig))
    drilldownList = drilldownConfig.get(drilldownFieldsName, section='drilldownFields')

#    logger.debug (infra.prpt("list",drilldownList))
    list = []
    for line in drilldownList.rsplit("\n"):
        if line == "":
            continue
        (name,number,boolean) = line.rsplit(',')
        if boolean == 'True':
            boolean = True
        else:
            boolean = False
        number = int (number)
        tupe = (name.strip(), number, boolean)
        list.append(tupe)
    return list

def okJSON():
    CRLF = "\r\n"
    ContentTypeJSON = "Content-Type: application/json"
    Ok = "HTTP/1.0 200 Ok" + CRLF
    okJSON = "HTTP/1.0 200 OK" + CRLF + \
             ContentTypeJSON + CRLF + \
             CRLF
    return okJSON

class Request(Observable):
    def __init__(self, configs):
        Observable.__init__(self)
        self.configs = configs

    def handleRequest(self, RequestURI='', **kwargs):
        Scheme, Netloc, Path, Query, Fragment = urlsplit(RequestURI)
        arguments = parse_qs(Query)
        query =  arguments.get('query', [''])[0]
        drilldownArguments = arguments.get('drilldown', [])
        extra = arguments.get('extra', [])
        parsetree, cqlQuery = self.any.parseQuery(query, drilldownArguments, extra)
        start = int(arguments.get('start', ['0'])[0])
        stop = int(arguments.get('stop', ['10'])[0])
        sortBy =  arguments.get('sortBy', [''])[0]
        sortDescending =  arguments.get('sortDescending', ['1'])[0]
        sortDescending = sortDescending == '1'
        total, recordIds = self.any.executeCQL(cqlAbstractSyntaxTree=parsetree, start=start, stop=stop, sortBy=sortBy, sortDescending=sortDescending)
        records = {}
        for recordId in recordIds:
             record = self.any.createRecord(recordId)
             records[recordId] = record
        docset = self.any.docsetFromQuery(cqlAbstractSyntaxTree=parsetree)
        drilldownConfig = self.configs['drilldown']
        drilldownFields =  getDrilldownFields(drilldownConfig, 'drilldownFields')
        drilldownResults = self.any.drilldown(docset, drilldownFields)
        drilldownresults = {}
        for fieldname, termCounts in drilldownResults:
            facet = []
            for term, count in termCounts:
                facet.append([str(term),str(count)])
            drilldownresults[fieldname] = facet
        multilevelResults = self.any.multiLevelDrilldown(docset, ['date', 'author'])
        multilevelresults = {}
        for (fieldName, levelField), termCounts in multilevelResults:
            facet = []
            for term, count in termCounts:
                facet.append([str(term),str(count)])
            multilevelresults[fieldName] = {'level':str(levelField), 'termCounts':facet} 
        response = {'total':total, 'recordIds':recordIds, 'records':records, 'drilldownResults':drilldownresults, 'multilevelResults':multilevelresults}
        yield okJSON()
        yield JSONEncoder().encode(response)

class Response(Observable):
    def __init__(self):
         Observable.__init__(self)

    def getResponse(self, query='', drilldownArguments=[], extra=[], dbserver='self', start=1, stop=11, sortBy='drilldown.norm.date.year', sortDescending='1'):
        json = '{}'
        if dbserver[0:4] != 'http':
            json = '{"diagnostic": "no http or https url"}'
        elif query:

            Query = [("query",query), ("start",str(start)), ("stop",str(stop)), ("sortBy",sortBy), ("sortDescending",sortDescending)]
            if drilldownArguments:
                 for drilldown in drilldownArguments:
                     Query.append(("drilldown", drilldown))
            if extra:
                 for e in extra:
                     Query.append(('extra', extra))
            request = '/JSON/searchresult?' + urllib.urlencode(Query)
            url = dbserver + request
            serverresponse = urllib.urlopen(url)
            json = serverresponse.read()
            #server = HTTPConnection("localhost", 6084)
            #server.connect()
            #server.request("GET", request)
            #serverresponse =  server.getresponse()
            #if serverresponse.status == OK:
            #    json = serverresponse.read()
            #else:
            #    json = '{}'
        else:
            json = '{"total":0}'
        return JSONDecoder().decode(json)
